//
// MoonCanvas.cpp
// Draw the moon and related info.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "MoonCanvas.h"

#include "MoonFeature.h"

// XXX next two should be removable:
#include <stdio.h>
#include <math.h>

#include <stdlib.h>

#define DEGMAX 100
#define DEGMIN  -5

int MoonCanvas::sDebugMode = 0;

MoonCanvas::MoonCanvas()
    : mMoonRadius(0), mMoonCenterX(-1), mMoonCenterY(-1),
      mMirrored(1), mZoomLevel(0),
      mCraterNum(0)
{
}

MoonCanvas::~MoonCanvas()
{
}

void MoonCanvas::Init()
{
    Zoom(0);

    MoonFeature::InitMoonFeatures(mMoonRadius);

    MoonFeature::SetCanvasSize(Width(), Height());
}

void MoonCanvas::ToggleMirrored()
{
    mMirrored = -mMirrored;
    Zoom(mZoomLevel);

    Paint();
}

void MoonCanvas::PaintFeature(MoonFeature* feature)
{
    feature->PaintFeature(this, mMoonCenterX, mMoonCenterY, mMoonRadius,
                          mMirrored, mImgLibrationE, mImgLibrationN);
}

void MoonCanvas::MousePress(int mouseX, int mouseY, int btnNum, int clickNum)
{
    // We're only interested in doubleclick events:
    if (clickNum <= 1) {
        mCraterNum = 0;
    }
#if 0
    // Need an alert/message method!
    else
        QMessageBox::information(this, "QMoon", mDescriptions,
                                 QMessageBox::Ok);
#endif
}

void MoonCanvas::PaintFeatures()
{
    int i;
    for (i=0; i < mCraterNum; ++i)
        mMatches[i]->PaintFeature(this, mMoonCenterX, mMoonCenterY,
                                  mMoonRadius,
                                  mMirrored, mImgLibrationE, mImgLibrationN);
}

void MoonCanvas::MouseRelease(int mouseX, int mouseY, int btnNum, int clickNum)
{
    double fx = (mouseX - mMoonCenterX) * mMirrored;
    double fy = mouseY - mMoonCenterY;

    // Translate that to moon coordinates
    double fLatitude = -asin(fy / mMoonRadius);
    double latitude = fLatitude * 180./M_PI - mImgLibrationN;
    double longitude = asin(fx / mMoonRadius / cos(fLatitude)) * 180./M_PI
                        - mImgLibrationE;
    if (sDebugMode)
        printf("Clicked at %dE, %dN\n", (int)latitude, (int)longitude);

    // Figure out which crater lives there
    double mindist = mMoonRadius*mMoonRadius;
    MoonFeature* nearfeature = 0;
    MoonFeature* feature;
    MoonFeature::InitLoop();

    while ((feature = MoonFeature::NextFeature()) != 0)
    {
        int isInside;
        double thisdist = feature->Distance(longitude, latitude, &isInside);
        if (isInside)
        {
            // For big maria, lots of things will be inside
            // and it can be hard to click exactly on an object.
            // So for big features, don't let "inside" stop us
            // from looking for a nearby crater.
            if (feature->GetSize() < 150)
            {
                mindist = 0;
                nearfeature = 0;
            }
            mMatches[mCraterNum++] = feature;
            if (mCraterNum >= MAX_OBJECTS_SHOWN-1)
                break;
            //PaintFeature(feature);
            if (sDebugMode)
                printf("Inside %s\n", feature->GetName());
        }
        else if (thisdist < mindist)
        {
            nearfeature = feature;
            mindist = thisdist;
        }
    }

    // Draw the nearby crater if we aren't inside of one:
    if (nearfeature)
    {
        if (mCraterNum < MAX_OBJECTS_SHOWN-1)
            mMatches[mCraterNum++] = nearfeature;

        if (sDebugMode)
            printf("Landed near %s\n", nearfeature->GetName());
        //PaintFeature(nearfeature);
    }

    Paint();
}

